#include "joystick.h"

#define WAIT6B         Delay1KTCYx(2)      // 168us
#define FP6B_SEL_L     LATC = 0x00
#define FP6B_SEL_H     LATC = 0x20


// Prottype　definition
void PrepareForRead6B(void);

/// -------------------------------------------------------
/// 読み取り準備（ポート設定など）
/// -------------------------------------------------------
void PrepareForRead6B(void)
{
    TRISC = 0b11011111;         // SEL出力、他は入力
    FP6B_SEL_H;
    WAIT_PORTSET;
}

void ReadPhaseData6B(unsigned char *phaseData)
{
    for(int i=0; i<8; i=i+2)
    {
        //PORTCbits.RC5 = 0;  // SEL = L
        FP6B_SEL_L;
        WAIT6B;
        phaseData[i  ] = (PORTC & 0x1f) | ((PORTCbits.RC6)? 0x20 : 0);

        //PORTCbits.RC5 = 1;  // SEL = H
        FP6B_SEL_H;
        WAIT6B;
        phaseData[i+1] = (PORTC & 0x1f) | ((PORTCbits.RC6)? 0x20 : 0);
    }
}


/// -------------------------------------------------------
/// 読み取り＆ジョイスティック出力データ作成
/// （ファイティングパッド6B）
/// -------------------------------------------------------
bool ReadFightingPad6B
    (char *devType, /*uint32_t*/int *buttons/*, char *hat, char *x, char *y, char *z, char *r */)
{
    bool retVal = true;
    unsigned char phaseData[8];
    static unsigned int counter_Failed = 0;
    bool state = true;

    if(*devType == DEVTYPE_UNKNOWN)
    {
        PrepareForRead6B();
    }

    // Read phaseData
    ReadPhaseData6B(phaseData);

    /*
    for(int i=0; i<8; i=i+2)
    {
        //PORTCbits.RC5 = 0;  // SEL = L
        FP6B_SEL_L;
        WAIT6B;
        phaseData[i  ] = (PORTC & 0x1f) | ((PORTCbits.RC6)? 0x20 : 0);

        //PORTCbits.RC5 = 1;  // SEL = H
        FP6B_SEL_H;
        WAIT6B;
        phaseData[i+1] = (PORTC & 0x1f) | ((PORTCbits.RC6)? 0x20 : 0);
    }
    */

    // Connection detection
    if(
       ((phaseData[0] & 0x0c) == 0x00) && ((phaseData[2] & 0x0c) == 0x00) &&
       ((phaseData[4] & 0x0f) == 0x00) && ((phaseData[6] & 0x0f) == 0x0f)
    )
    {
        state = true;
    }
    else
    {
        state = false;
    }



    if(state)
    {
        counter_Failed = 0;
        *devType = DEVTYPE_MD;

        //Buttons
        *buttons = 0x00000000 |
            ((phaseData[0]&0b010000) ? 0 : BTNBIT_01) |  // A
            ((phaseData[1]&0b010000) ? 0 : BTNBIT_02) |  // B
            ((phaseData[1]&0b100000) ? 0 : BTNBIT_03) |  // C
            ((phaseData[5]&0b000100) ? 0 : BTNBIT_04) |  // X
            ((phaseData[5]&0b000010) ? 0 : BTNBIT_05) |  // Y
            ((phaseData[5]&0b000001) ? 0 : BTNBIT_06) |  // Z
            ((phaseData[5]&0b001000) ? 0 : BTNBIT_09) |  // Mode
            ((phaseData[0]&0b100000) ? 0 : BTNBIT_10) ;  // Start

        if(
            (*buttons & BTNBIT_01) &&
            (*buttons & BTNBIT_02) &&
            (*buttons & BTNBIT_03) &&
            (*buttons & BTNBIT_09) &&
            (*buttons & BTNBIT_10)
        )
        {
            *buttons = (*buttons | BTNBIT_13) &
            (~(BTNBIT_01 |
               BTNBIT_02 |
               BTNBIT_03 |
               BTNBIT_09 |
               BTNBIT_10 ));
        }

        Set_AnalogAndHatValue(
            (phaseData[1]&0b001000),
            (phaseData[1]&0b000100),
            (phaseData[0]&0b000001),
            (phaseData[0]&0b000010)
            /*,hat, x, y, z, r*/);
    }
    else
    {
        counter_Failed++;
        if(counter_Failed>10)
        {
            counter_Failed = 0;

            *devType = DEVTYPE_UNKNOWN;

            //Buttons
            *buttons = 0x00000000 |
                ((PORTCbits.RC6) ? 0 : BTNBIT_01) |  // A
                ((PORTCbits.RC4) ? 0 : BTNBIT_02) ;  // B

            Set_AnalogAndHatValue(
                (PORTCbits.RC3),
                (PORTCbits.RC2),
                (PORTCbits.RC0),
                (PORTCbits.RC1)
                /*,hat, x, y, z, r*/);
        }

    }

    //Delay10KTCYx(2); // 1.6ms

    return retVal;
}
